/*
 * Copyright (c) 2013 CNRS/LAAS
 *
 * Permission to use, copy, modify, and/or distribute this software for any
 * purpose with or without fee is hereby granted, provided that the above
 * copyright notice and this permission notice appear in all copies.
 *
 * THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES
 * WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR
 * ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
 * WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN
 * ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF
 * OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 */

#ifndef _DAQFLEX_H
#define _DAQFLEX_H

/* Maximum number of channels for USB-201 devices*/
#define DAQFLEX_USB201_MAX_NUM_CHANS 8

/*
 * mcc daqflex descriptor
 *
 * XXX more or less specifi to USB-201 devices. should be split up
 * in generic daqflex part and device dependent part
 */
struct mccdevice {
	/* libusb specific */
	libusb_device_handle *usbDev;
	libusb_device **devices;
	uint8_t inEndPoint;
	uint8_t outEndPoint;
	uint16_t bulkPacketSize;
	/* USB-201 specific part */
	uint16_t resolution;	/* mask of active bits in data values  */
	int lowChannel;		/* lowest channel to read */
	int highChannel;	/* highest channel to read */
	int numChans;		/* number of selected channels */
	int bufSize;		/* buffer size (in 16 bit values) */
	int16_t *buffer;	/* buffer */
	int readPtr;		/* read pointer */
	int writePtr;		/* write pointer */
	int numValues;		/* number of values in buffer */
	double slope[DAQFLEX_USB201_MAX_NUM_CHANS]; /* calibration */
	double offset[DAQFLEX_USB201_MAX_NUM_CHANS]; /* data */
};

/* prototypes - see src/daqflex.c for descriptions */
extern struct mccdevice *daqflexOpen(void);
extern void daqflexClose(struct mccdevice *);
extern int daqflexSendMessage(struct mccdevice *, char *, char *, int);
extern void daqflexFlushInputData(struct mccdevice *);
extern int daqflexReadPacket(struct mccdevice *, unsigned char *, int);

/* USB-201 specific functions */
extern int daqflexUSB201Init(struct mccdevice *, int, int, double);
extern int daqflexUSB201ReadData(struct mccdevice *, double *, int);

#endif
