/*
 * Copyright (c) 2013 CNRS/LAAS
 *
 * Permission to use, copy, modify, and/or distribute this software for any
 * purpose with or without fee is hereby granted, provided that the above
 * copyright notice and this permission notice appear in all copies.
 *
 * THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES
 * WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR
 * ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
 * WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN
 * ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF
 * OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 */

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#include <signal.h>
#include <stdio.h>
#include <stdint.h>
#include <stdlib.h>
#include <time.h>

#include <libusb.h>
#include <daqflex/daqflex.h>

int done = 0;

/*
 * Ctrl-C handler
 */
void
ctrlc(int sig)
{
	done++;
}

/*
 * Difference of timespec result = x - y
 */
int
timespec_subtract (result, x, y)
	struct timespec *result, *x, *y;
{
	/* Perform the carry for the later subtraction by updating y. */
	if (x->tv_nsec < y->tv_nsec) {
		int nsec = (y->tv_nsec - x->tv_nsec) / 1000000000 + 1;
		y->tv_nsec -= 1000000000 * nsec;
		y->tv_sec += nsec;
	}
	if (x->tv_nsec - y->tv_nsec > 1000000000) {
		int nsec = (x->tv_nsec - y->tv_nsec) / 1000000000;
		y->tv_nsec += 1000000000 * nsec;
		y->tv_sec -= nsec;
	}
	/* Compute the time remaining to wait.
	   tv_nsec is certainly positive. */
	result->tv_sec = x->tv_sec - y->tv_sec;
	result->tv_nsec = x->tv_nsec - y->tv_nsec;

	/* Return 1 if result is negative. */
	return x->tv_sec < y->tv_sec;
}


int
main(int argc, char *argv[])
{
	struct timespec c1, c2, c3;
	struct mccdevice *dev;
	char response[64];
	double data[8];
	int i = 0, num;

	/* Connect to the DAQ device */
	dev = daqflexOpen();
	if (dev == NULL)
		exit(2);

	daqflexFlushInputData(dev);

	/* Display the serial number - just for fun for now */
	daqflexSendMessage(dev, "?DEV:MFGSER",
	    response, sizeof(response));
	printf("'%s'\n", response);

	/* Init the USB-201 device: acquire channels 0 to 7 at 1kHz */
	daqflexUSB201Init(dev, 0, 7, 1000.0);

	/* Start an infinite loop of aquisition */
	daqflexSendMessage(dev, "AISCAN:SAMPLES=0",
	    response, sizeof(response));
	daqflexSendMessage(dev, "AISCAN:START",
	    response, sizeof(response));
	printf("'%s'\n", response);

	/* Read and display the data in a loop */
	signal(SIGINT, ctrlc);
	clock_gettime(CLOCK_REALTIME, &c1);
	while (!done) {
		num = daqflexUSB201ReadData(dev, data, 8);
		if (num != 8)
			break;
#if 1
		printf("%5d %.4f %.4f %.4f %.4f %.4f %.4f %.4f %.4f\n", i,
		    data[0], data[1], data[2], data[3], data[4], data[5],
		    data[6], data[7]);
#endif
		i++;
	}
	clock_gettime(CLOCK_REALTIME, &c2);
	timespec_subtract(&c3, &c2, &c1);
	printf("got %d frames in %ld.%09ld sec\n", i,
	    (long)c3.tv_sec, (long)c3.tv_nsec);
	/* Stop acquisition */
	daqflexSendMessage(dev, "AISCAN:STOP", response, sizeof(response));
	printf("'%s'\n", response);
	/* Free resources */
	daqflexClose(dev);
	return 0;
}
